jansvoboda11 updated this revision to Diff 338034.
jansvoboda11 added a comment.

Rebase


Repository:
  rG LLVM Github Monorepo

CHANGES SINCE LAST ACTION
  https://reviews.llvm.org/D100460/new/

https://reviews.llvm.org/D100460

Files:
  clang-tools-extra/clangd/TUScheduler.cpp
  clang-tools-extra/clangd/tool/Check.cpp
  clang/include/clang/Frontend/CompilerInvocation.h
  clang/lib/ARCMigrate/ARCMT.cpp
  clang/lib/Frontend/ASTUnit.cpp
  clang/lib/Frontend/ChainedIncludesSource.cpp
  clang/lib/Frontend/CompilerInstance.cpp
  clang/lib/Frontend/PrecompiledPreamble.cpp
  clang/lib/Frontend/Rewrite/FrontendActions.cpp
  clang/lib/StaticAnalyzer/Frontend/ModelInjector.cpp
  clang/unittests/Frontend/CompilerInvocationTest.cpp

Index: clang/lib/StaticAnalyzer/Frontend/ModelInjector.cpp
===================================================================
--- clang/lib/StaticAnalyzer/Frontend/ModelInjector.cpp
+++ clang/lib/StaticAnalyzer/Frontend/ModelInjector.cpp
@@ -64,7 +64,8 @@
     return;
   }
 
-  auto Invocation = std::make_shared<CompilerInvocation>(CI.getInvocation());
+  auto Invocation =
+      std::make_shared<CompilerInvocation>(CI.getInvocation().clone());
 
   FrontendOptions &FrontendOpts = Invocation->getFrontendOpts();
   InputKind IK = Language::CXX; // FIXME
Index: clang/lib/Frontend/Rewrite/FrontendActions.cpp
===================================================================
--- clang/lib/Frontend/Rewrite/FrontendActions.cpp
+++ clang/lib/Frontend/Rewrite/FrontendActions.cpp
@@ -244,7 +244,7 @@
     CompilerInstance Instance(CI.getPCHContainerOperations(),
                               &CI.getModuleCache());
     Instance.setInvocation(
-        std::make_shared<CompilerInvocation>(CI.getInvocation()));
+        std::make_shared<CompilerInvocation>(CI.getInvocation().clone()));
     Instance.createDiagnostics(
         new ForwardingDiagnosticConsumer(CI.getDiagnosticClient()),
         /*ShouldOwnClient=*/true);
Index: clang/lib/Frontend/PrecompiledPreamble.cpp
===================================================================
--- clang/lib/Frontend/PrecompiledPreamble.cpp
+++ clang/lib/Frontend/PrecompiledPreamble.cpp
@@ -317,7 +317,8 @@
     PreambleCallbacks &Callbacks) {
   assert(VFS && "VFS is null");
 
-  auto PreambleInvocation = std::make_shared<CompilerInvocation>(Invocation);
+  auto PreambleInvocation =
+      std::make_shared<CompilerInvocation>(Invocation.clone());
   FrontendOptions &FrontendOpts = PreambleInvocation->getFrontendOpts();
   PreprocessorOptions &PreprocessorOpts =
       PreambleInvocation->getPreprocessorOpts();
@@ -501,7 +502,8 @@
       Bounds.Size <= MainFileBuffer.getBufferSize() &&
       "Buffer is too large. Bounds were calculated from a different buffer?");
 
-  auto PreambleInvocation = std::make_shared<CompilerInvocation>(Invocation);
+  auto PreambleInvocation =
+      std::make_shared<CompilerInvocation>(Invocation.clone());
   PreprocessorOptions &PreprocessorOpts =
       PreambleInvocation->getPreprocessorOpts();
 
Index: clang/lib/Frontend/CompilerInstance.cpp
===================================================================
--- clang/lib/Frontend/CompilerInstance.cpp
+++ clang/lib/Frontend/CompilerInstance.cpp
@@ -1041,8 +1041,8 @@
   llvm::TimeTraceScope TimeScope("Module Compile", ModuleName);
 
   // Construct a compiler invocation for creating this module.
-  auto Invocation =
-      std::make_shared<CompilerInvocation>(ImportingInstance.getInvocation());
+  auto Invocation = std::make_shared<CompilerInvocation>(
+      ImportingInstance.getInvocation().clone());
 
   PreprocessorOptions &PPOpts = Invocation->getPreprocessorOpts();
 
Index: clang/lib/Frontend/ChainedIncludesSource.cpp
===================================================================
--- clang/lib/Frontend/ChainedIncludesSource.cpp
+++ clang/lib/Frontend/ChainedIncludesSource.cpp
@@ -124,8 +124,8 @@
 
   for (unsigned i = 0, e = includes.size(); i != e; ++i) {
     bool firstInclude = (i == 0);
-    std::unique_ptr<CompilerInvocation> CInvok;
-    CInvok.reset(new CompilerInvocation(CI.getInvocation()));
+    auto CInvok =
+        std::make_unique<CompilerInvocation>(CI.getInvocation().clone());
 
     CInvok->getPreprocessorOpts().ChainedIncludes.clear();
     CInvok->getPreprocessorOpts().ImplicitPCHInclude.clear();
Index: clang/lib/Frontend/ASTUnit.cpp
===================================================================
--- clang/lib/Frontend/ASTUnit.cpp
+++ clang/lib/Frontend/ASTUnit.cpp
@@ -1104,7 +1104,7 @@
     assert(VFS == &FileMgr->getVirtualFileSystem() &&
            "VFS passed to Parse and VFS in FileMgr are different");
 
-  auto CCInvocation = std::make_shared<CompilerInvocation>(*Invocation);
+  auto CCInvocation = std::make_shared<CompilerInvocation>(Invocation->clone());
   if (OverrideMainBuffer) {
     assert(Preamble &&
            "No preamble was built, but OverrideMainBuffer is not null");
@@ -2131,7 +2131,7 @@
   CompletionTimer.setOutput("Code completion @ " + File + ":" +
                             Twine(Line) + ":" + Twine(Column));
 
-  auto CCInvocation = std::make_shared<CompilerInvocation>(*Invocation);
+  auto CCInvocation = std::make_shared<CompilerInvocation>(Invocation->clone());
 
   FrontendOptions &FrontendOpts = CCInvocation->getFrontendOpts();
   CodeCompleteOptions &CodeCompleteOpts = FrontendOpts.CodeCompleteOpts;
Index: clang/lib/ARCMigrate/ARCMT.cpp
===================================================================
--- clang/lib/ARCMigrate/ARCMT.cpp
+++ clang/lib/ARCMigrate/ARCMT.cpp
@@ -173,8 +173,7 @@
 static CompilerInvocation *
 createInvocationForMigration(CompilerInvocation &origCI,
                              const PCHContainerReader &PCHContainerRdr) {
-  std::unique_ptr<CompilerInvocation> CInvok;
-  CInvok.reset(new CompilerInvocation(origCI));
+  auto CInvok = std::make_unique<CompilerInvocation>(origCI.clone());
   PreprocessorOptions &PPOpts = CInvok->getPreprocessorOpts();
   if (!PPOpts.ImplicitPCHInclude.empty()) {
     // We can't use a PCH because it was likely built in non-ARC mode and we
@@ -346,13 +345,13 @@
   LangOptions::GCMode OrigGCMode = origCI.getLangOpts()->getGC();
 
   // Make sure checking is successful first.
-  CompilerInvocation CInvokForCheck(origCI);
+  CompilerInvocation CInvokForCheck(origCI.clone());
   if (arcmt::checkForManualIssues(CInvokForCheck, Input, PCHContainerOps,
                                   DiagClient, emitPremigrationARCErrors,
                                   plistOut))
     return true;
 
-  CompilerInvocation CInvok(origCI);
+  CompilerInvocation CInvok(origCI.clone());
   CInvok.getFrontendOpts().Inputs.clear();
   CInvok.getFrontendOpts().Inputs.push_back(Input);
 
@@ -510,7 +509,7 @@
     const CompilerInvocation &CI,
     std::shared_ptr<PCHContainerOperations> PCHContainerOps,
     DiagnosticConsumer *diagClient, StringRef outputDir)
-    : OrigCI(CI), PCHContainerOps(std::move(PCHContainerOps)),
+    : OrigCI(CI.clone()), PCHContainerOps(std::move(PCHContainerOps)),
       DiagClient(diagClient), HadARCErrors(false) {
   if (!outputDir.empty()) {
     IntrusiveRefCntPtr<DiagnosticIDs> DiagID(new DiagnosticIDs());
Index: clang/include/clang/Frontend/CompilerInvocation.h
===================================================================
--- clang/include/clang/Frontend/CompilerInvocation.h
+++ clang/include/clang/Frontend/CompilerInvocation.h
@@ -185,7 +185,20 @@
 /// options, the warning flags, and so on.
 class CompilerInvocation : public CompilerInvocationRefBase,
                            public CompilerInvocationValueBase {
+  /// Implementation detail of clone().
+  CompilerInvocation(const CompilerInvocation &) = default;
+
 public:
+  CompilerInvocation() = default;
+  CompilerInvocation(CompilerInvocation &&) = default;
+  CompilerInvocation &operator=(CompilerInvocation &&) = default;
+  CompilerInvocation &operator=(const CompilerInvocation &) = delete;
+
+  /// Create a deep copy.
+  /// Also copies CompilerInvocationRefBase and its members behind
+  /// reference-counted pointers.
+  CompilerInvocation clone() const { return CompilerInvocation(*this); }
+
   /// Create a compiler invocation from a list of input options.
   /// \returns true on success.
   ///
Index: clang-tools-extra/clangd/tool/Check.cpp
===================================================================
--- clang-tools-extra/clangd/tool/Check.cpp
+++ clang-tools-extra/clangd/tool/Check.cpp
@@ -159,7 +159,7 @@
   bool buildAST() {
     log("Building preamble...");
     Preamble =
-        buildPreamble(File, *Invocation, Inputs, /*StoreInMemory=*/true,
+        buildPreamble(File, Invocation->clone(), Inputs, /*StoreInMemory=*/true,
                       [&](ASTContext &Ctx, std::shared_ptr<Preprocessor> PP,
                           const CanonicalIncludes &Includes) {
                         if (!Opts.BuildDynamicSymbolIndex)
Index: clang-tools-extra/clangd/TUScheduler.cpp
===================================================================
--- clang-tools-extra/clangd/TUScheduler.cpp
+++ clang-tools-extra/clangd/TUScheduler.cpp
@@ -925,7 +925,7 @@
   }
 
   LatestBuild = clang::clangd::buildPreamble(
-      FileName, *Req.CI, Inputs, StoreInMemory,
+      FileName, Req.CI->clone(), Inputs, StoreInMemory,
       [this, Version(Inputs.Version)](ASTContext &Ctx,
                                       std::shared_ptr<clang::Preprocessor> PP,
                                       const CanonicalIncludes &CanonIncludes) {
_______________________________________________
cfe-commits mailing list
cfe-commits@lists.llvm.org
https://lists.llvm.org/cgi-bin/mailman/listinfo/cfe-commits

Reply via email to
Index: clang/unittests/Frontend/CompilerInvocationTest.cpp
===================================================================
--- clang/unittests/Frontend/CompilerInvocationTest.cpp
+++ clang/unittests/Frontend/CompilerInvocationTest.cpp
@@ -97,13 +97,24 @@
   ASSERT_THAT(Array, ContainsN(StrEq("x"), 2));
 }
 
 // Copy constructor/assignment perform deep copy of reference-counted pointers.
 
-TEST(CompilerInvocationTest, DeepCopyConstructor) {
+TEST(CompilerInvocationTest, CloneWithMoveConstructor) {
   CompilerInvocation A;
   A.getAnalyzerOpts()->Config["Key"] = "Old";
 
-  CompilerInvocation B(A);
+  CompilerInvocation B(A.clone());
+  B.getAnalyzerOpts()->Config["Key"] = "New";
+
+  ASSERT_EQ(A.getAnalyzerOpts()->Config["Key"], "Old");
+}
+
+TEST(CompilerInvocationTest, CloneWithMoveAssignment) {
+  CompilerInvocation A;
+  A.getAnalyzerOpts()->Config["Key"] = "Old";
+
+  CompilerInvocation B;
+  B = A.clone();
   B.getAnalyzerOpts()->Config["Key"] = "New";
 
   ASSERT_EQ(A.getAnalyzerOpts()->Config["Key"], "Old");
@@ -120,6 +131,11 @@
   ASSERT_EQ(A.getAnalyzerOpts()->Config["Key"], "Old");
 }
 
+TEST(CompilerInvocationTest, NotCopyConstructibleNorAssignable) {
+  ASSERT_FALSE(std::is_copy_constructible<CompilerInvocation>::value);
+  ASSERT_FALSE(std::is_copy_assignable<CompilerInvocation>::value);
+}
+
 // Boolean option with a keypath that defaults to true.
 // The only flag with a negative spelling can set the keypath to false.
 
@@ -97,13 +97,24 @@
 }
 
-// Copy constructor/assignment perform deep copy of reference-counted pointers.
+// Clone performs deep copy of reference-counted pointers.
 